<?php
/**
 * Application level Controller
 *
 * This file is application-wide controller file. You can put all
 * application-wide controller-related methods here.
 *
 * CakePHP(tm) : Rapid Development Framework (http://cakephp.org)
 * Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright     Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 * @link          http://cakephp.org CakePHP(tm) Project
 * @package       app.Controller
 * @since         CakePHP(tm) v 0.2.9
 * @license       http://www.opensource.org/licenses/mit-license.php MIT License
 */

App::uses('Controller', 'Controller');
App::import('Config', 'Constants');

/**
 * Application Controller
 *
 * Add your application-wide methods in the class below, your controllers
 * will inherit them.
 *
 * @package		app.Controller
 * @link		http://book.cakephp.org/2.0/en/controllers.html#the-app-controller
 */
class AppController extends Controller {

	public $components = array(
		'Session',
		'Auth' => array(
			'loginRedirect' => array(
				'controller' => 'pages',
				'action' => 'display',
				'home'
			),
			'logoutRedirect' => array(
				'controller' => 'users',
				'action' => 'login'
			),
			'authenticate' => array(
				'Form' => array(
					'passwordHasher' => array(
						'className' => 'Simple',
						'hashType' => 'sha256'
					),
					'fields' => array('username' => 'email')
				)
			),
			'authorize' => array('Controller'),
		)
	);

	public function beforeFilter() {
		$this->Auth->allow();


		//Check version
		if ((!empty($this->data['ver']) && $this->data['ver'] != Constants::LAST_VERSION) ||
			(!empty($this->request->query['ver']) && $this->request->query['ver'] != Constants::LAST_VERSION)) {

			$this->redirect('/users/error_version.json');
		}
	}

	public function sendPush($token, $title, $messageText, $type = null, $id = null) {
		if(!is_array($token)) {
			$token = array($token);
		}

		App::uses('HttpSocket', 'Network/Http');

		$HttpSocket = new HttpSocket(array('ssl_verify_host' => false));
		$data = array(
			'registration_ids' => $token,
			'collapse_key' => null,
			'data' => array(
				'title' => $title,
				'message' => $messageText,
				'type' => $type,
				'id' => $id
			)
		);
		$result = $HttpSocket->post(
			'https://android.googleapis.com/gcm/send',
			json_encode($data),
			array(
				'header' => array(
					'Authorization' => 'key='.Constants::GCM_KEY,
					'Content-Type' => 'application/json'
				)
			)
		);

		if($result->code !== "200") {
			return false;
		}
		else {
			return json_decode($result->body, TRUE);
		}
	}
}
