<?php
App::uses('AppController', 'Controller');
/**
 * Baskets Controller
 *
 */
class BasketsController extends AppController {

/**
 * Scaffold
 *
 * @var mixed
 */
	public $scaffold;

	public $components = array('RequestHandler');
	public $uses = array('Basket', 'Client', 'User', 'Activity', 'Offer', 'Account', 'Contract', 'Shop');

	public function index()
	{
		$user_id = $this->request->query['user_id'];
		$token = $this->request->query['token'];

		$user = $this->User->find('first', array('conditions' => array('id' => $user_id, 'token' => $token)));
		if (empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		if (isset($this->request->query['shop_view'])) {
			$shop_view = $this->request->query['shop_view'];
		} else {
			$shop_view = false;
		}

		if(!empty($user['Shop']) && $shop_view == true ) {
			$this->Activity->recursive = -1;
			$activities = $this->Activity->find('all', array(
				'joins' => array( array(
					'table' => 'activity_shop',
					'alias' => 'ActivityShop',
					'conditions' => array(
						'shop_id' => $user['Shop'][0]['id'],
						'ActivityShop.activity_id = Activity.id',
					)
				)),
				'fields' => array('Activity.id')
			));

			$activities_id = array();
			foreach($activities as $activity) {
				$activities_id[] = $activity['Activity']['id'];
			}

			$this->Basket->recursive = -1;
			$this->Basket->Behaviors->load('Containable');
			$baskets = $this->Basket->find('all', array(
				'conditions' => array(
					'Activity.id' => $activities_id,
					'active' => true,
					'state' => Constants::STATE_ON_SALE
				),
				'contain' => array('Activity.name', 'Activity.icon', 'Activity.id', 'Client.User.username', 'Offer.shop_id', 'Offer.status'),
				'order' => array('Basket.created desc')
			));

			foreach($baskets as $key => $basket) {
				foreach($baskets[$key]['Offer'] as $offer) {
					if($offer['shop_id'] == $user['Shop'][0]['id'] && $offer['status'] == Constants::STATE_ON_SALE) {
						unset($baskets[$key]);
					}
				}
			}

			$baskets = array_values($baskets);
		} else {
			$client = $this->Client->findByUserId($user_id);
			if (!empty($client)) {
				$this->Basket->recursive = -1;
				$this->Basket->Behaviors->load('Containable');
				$baskets = $this->Basket->find('all', array(
					'conditions' => array(
						'client_id' => $client['Client']['id'],
						'active' => true
					),
					'contain' => array('Activity.name','Offer.id', 'Offer.status', 'Activity.icon', 'Activity.id', 'Contract', 'Contract.Shop'),
					'order' => array('Basket.state asc'),
				));

				// Contem les ofertes pendents
				foreach($baskets as $key => $basket) {
					$baskets[$key]['pendingOffers'] = 0;
					foreach($basket['Offer'] as $offer) {
						if($offer['status'] == Constants::STATE_ON_SALE) {
							$baskets[$key]['pendingOffers']++;
						}
					}
				}
			} else {
				$baskets = array();
			}
		}

		$this->set(array(
			'result' => Constants::RESULT_OK,
			'baskets' => $baskets,
			'_serialize' => array('result', 'baskets')
		));
	}

	public function view($basket_id) {

		$user_id = $this->request->query['user_id'];
		$token = $this->request->query['token'];

		$user = $this->User->find('first', array('conditions' => array('id' => $user_id, 'token' => $token)));

		if(empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

//		$basket = $this->Basket->findById($basket_id);

		$contract = $this->Contract->findByBasketId($basket_id);

		if(empty($contract)) {
			$this->Basket->Behaviors->load('Containable');
			$basket = $this->Basket->find('first', array(
				'conditions' => array('Basket.id' => $basket_id),
				'contain' => array('Activity', 'Client.User.username','Offer', 'Offer.Shop', 'Offer.Shop.Activity', 'Offer.Shop.User.email')
			));
		} else {
			$this->Basket->Behaviors->load('Containable');
			$basket = $this->Basket->find('first', array(
				'conditions' => array('Basket.id' => $basket_id),
				'contain' => array('Activity', 'Client.User.username', 'Contract')
			));
		}


		$this->set(array(
			'result' => Constants::RESULT_OK,
			'basket' => $basket,
			'_serialize' => array('result', 'basket')
		));

	}

	public function add() {

		$user_id = $this->request->data['user_id'];
		$token = $this->request->data['token'];

		$this->User->recursive = -1;
		$this->User->Behaviors->load('Containable');
		$user = $this->User->find('first',
			array(
				'conditions' => array('id' => $user_id, 'token' => $token),
				'contain' => array('Client', 'Client.Account')
			)
		);

		if(empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		if(!isset($this->request->data['amount']) || $this->request->data['amount'] <= 0) {
			$this->set(array(
				'result' => Constants::ERROR_AMOUNT,
				'message' => 'La quantitat entrada no es vàlida.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		if($user['Client'][0]['Account']['balance'] < $this->request->data['amount']) {
			$this->set(array(
				'result' => Constants::ERROR_GENERIC,
				'account' => $user['Client'][0]['Account'],
				'message' => __('No tens prous diners, com a màxim pots crear una cistella de '.$user['Client'][0]['Account']['balance'].' eurakos.'),
				'_serialize' => array('result', 'message', 'account')
			));
			return;
		} else {
			$account = array();
			$account['Account']['id'] = $user['Client'][0]['Account']['id'];
			$account['Account']['balance'] = $user['Client'][0]['Account']['balance'] - $this->request->data['amount'];
			$account['Account']['locked_balance'] = $user['Client'][0]['Account']['locked_balance'] + $this->request->data['amount'];

			if($this->Account->save($account)) {

				$basket = array();
				$basket['Basket']['activity_id'] = $this->request->data['activity'];
				$basket['Basket']['duration'] = $this->request->data['duration'];
				$basket['Basket']['amount'] = $this->request->data['amount'];
				$basket['Basket']['state'] = Constants::STATE_ON_SALE;
				$basket['Basket']['active'] = 1;
				$basket['Basket']['client_id'] = $this->Client->findByUserId($user_id)['Client']['id'];

				if($this->Basket->save($basket)) {

					// Enviem una notifiació a les botigues amb aquella activitat
					$this->Shop->recursive = -1;
					$this->Shop->Behaviors->load('Containable');
					$shops = $this->Shop->find('all', array(
						'joins' => array(
							array(
								'table' => 'activity_shop',
								'alias' => 'ActivityShop',
								'type' => 'left',
								'conditions' => array(
									'ActivityShop.id' =>  $this->request->data['activity'],
									'ActivityShop.shop_id = Shop.id'
								)
							)
						),
						'contain' => array(
							'User.gcmRegId'
						)
					));

					foreach($shops as $shop) {
						if ($shop['User']['gcmRegId'] != null) {
							$this->sendPush($shop['User']['gcmRegId'],
								__("Hi ha una cistella nova"),
								__('Un client acaba de crear una cistella nova que et pot interessar, entra per veure-la!.'),
								Constants::NOTIFICATION_NEW_BASKET
							);
						}
					}

					$this->set(array(
						'result' => Constants::RESULT_OK,
						'shops' => $shops,
						'id' => $this->Basket->getLastInsertID(),
						'_serialize' => array('result', 'id', 'shops')
					));
				} else {

					$account['Account']['balance'] = $user['Client'][0]['Account']['balance'];
					$account['Account']['locked_balance'] = $user['Client'][0]['Account']['locked_balance'];

					$this->Account->save($account);

					$this->set(array(
						'result' => Constants::ERROR_GENERIC,
						'message' => 'Error saving the basket.',
						'_serialize' => array('result', 'message')
					));
				}
			}
		}
	}

	public function sendNotification() {

		$user = $this->User->findById(76);

		if ($user['User']['gcmRegId'] != null) {
			$this->sendPush($user['User']['gcmRegId'],
				__("Hi ha una cistella nova"),
				__('Un client acaba de crear una cistella nova que et pot interessar, entra per veure-la!.'),
				Constants::NOTIFICATION_NEW_BASKET
			);

			$this->set(array(
				'result' => Constants::RESULT_OK,
				'_serialize' => array('result')
			));
		}
	}


	public function delete($basket_id) {

		$user_id = $this->request->query['user_id'];
		$token = $this->request->query['token'];

		$this->User->recursive = -1;
		$this->User->Behaviors->load('Containable');
		$user = $this->User->find('first',
			array(
				'conditions' => array('id' => $user_id, 'token' => $token),
				'contain' => array('Client', 'Client.Account')
			)
		);

		if(empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		$basket = $this->Basket->find('first', array(
			'conditions' => array(
				'Basket.id' => $basket_id,
				'active' => 1
			)
		));

		if(!empty($basket)) {
			if(empty($basket['Contract'])) {
				foreach($basket['Offer'] as $offer) {
					$this->Offer->id = $offer['id'];
					$this->Offer->saveField('active', 0);
				}

				$this->Basket->id = $basket['Basket']['id'];
				$this->Basket->saveField('active', 0);

				// Unlock the amount
				$account = array();
				$account['Account']['id'] = $user['Client'][0]['Account']['id'];
				$account['Account']['balance'] = $user['Client'][0]['Account']['balance'] + $basket['Basket']['amount'];
				$account['Account']['locked_balance'] = $user['Client'][0]['Account']['locked_balance'] - $basket['Basket']['amount'];

				$this->Account->save($account);

				$this->set(array(
					'result' => Constants::RESULT_OK,
					'_serialize' => array('result')
				));

			} else {
				$this->set(array(
					'result' => Constants::ERROR_GENERIC,
					'message' => __('No es pot eliminar una cistella que tingui una oferta acceptada.'),
					'_serialize' => array('result', 'message')
				));
			}
		} else {
			$this->set(array(
				'result' => Constants::ERROR_GENERIC,
				'message' => __('No s\'ha trobat la cistella'),
				'_serialize' => array('result', 'message')
			));
		}
	}
}
