<?php
App::uses('AppController', 'Controller');
App::import('Controller', 'Extracts');
/**
 * Offers Controller
 *
 */
class OffersController extends AppController {

/**
 * Scaffold
 *
 * @var mixed
 */
	public $scaffold;

	public $components = array('RequestHandler');
	public $uses = array('Offer', 'User', 'Client', 'Contract', 'Account', 'Shop', 'Basket');

	public function index()
	{
		$user_id = $this->request->query['user_id'];
		$token = $this->request->query['token'];

		$user = $this->User->find('first', array('conditions' => array('id' => $user_id, 'token' => $token)));

		if (empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		$this->Offer->recursive = -1;
		$this->Offer->Behaviors->load('Containable');
		$offers = $this->Offer->find('all', array(
			'conditions' => array(
				'Offer.shop_id' => $user['Shop'][0]['id'],
				'Offer.active' => true,
				'Offer.status' => array(Constants::STATE_ON_SALE, Constants::STATE_CANCELED, Constants::STATE_REFUSED)
			),
			'order' => array('status asc '),
			'limit' => 30,
			'contain' => array('Basket.amount', 'Basket.Client.User.username')
		));

		$this->set(array(
			'result' => Constants::RESULT_OK,
			'offers' => $offers,
			'_serialize' => array('result', 'offers')
		));
	}

	public function add()
	{
		$user_id = $this->request->data['user_id'];
		$token = $this->request->data['token'];

		$user = $this->User->find('first', array('conditions' => array('id' => $user_id, 'token' => $token)));

		if (empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		$basket_id = $this->request->data['basket_id'];

		$offer = $this->Offer->create();
		$offer['Offer']['basket_id'] = $basket_id;
		$offer['Offer']['shop_id'] = $user['Shop'][0]['id'];
		$offer['Offer']['percentage'] = $this->request->data['percentage'];
		$offer['Offer']['status'] = 0;
		$offer['Offer']['active'] = true;


		if($this->Offer->save($offer)) {

			// Enviem la notificacio a l'usuari de la cistella
			$this->Client->recursive = -1;
			$this->Client->Behaviors->load('Containable');
			$basket_owner = $this->Client->find('first', array(
				'joins' => array(
					array(
						'table' => 'basket',
						'alias' => 'Basket',
						'conditions' => array(
							'Basket.id' => $basket_id
						)
					)
				),
				'conditions' => array('Client.id = Basket.client_id'),
				'contain' => array('User')
			));

			if($basket_owner['User']['gcmRegId'] != null) {
				$this->sendPush($basket_owner['User']['gcmRegId'],
					__("Tens una nova oferta"),
					__('Una botiga acaba de fer una oferta a la teva cistella, comprova quants diners estalviaràs!'),
					Constants::NOTIFICATION_NEW_OFFER
				);
			}


			$this->set(array(
				'result' => Constants::RESULT_OK,
				'_serialize' => array('result')
			));
		} else {
			$this->set(array(
				'result' => Constants::ERROR_GENERIC,
				'message' => __('Ho sentim, no s\'ha pogut crear l\'oferta, torna-ho a provar siusplau.'),
				'_serialize' => array('result', 'message')
			));
		}
	}

	public function accept($id) {

		$user_id = $this->request->data['user_id'];
		$token = $this->request->data['token'];

		$user = $this->User->find('first', array('conditions' => array('id' => $user_id, 'token' => $token)));

		if (empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		$this->Offer->recursive = -1;
		$this->Offer->Behaviors->load('Containable');
		$offer = $this->Offer->find('first', array(
			'conditions' => array(
				'Offer.id' => $id,
				'Offer.active' => true,
				'Offer.status' => Constants::STATE_ON_SALE),
			'contain' => array('Basket', 'Shop.Account', 'Basket.Client.Account')
		));

		if(!empty($offer)) {
			$this->Offer->id = $offer['Offer']['id'];
			$this->Offer->saveField('status', Constants::STATE_SOLD);

			$this->Basket->id = $offer['Offer']['basket_id'];
			$this->Basket->saveField('state', Constants::STATE_SOLD);

			// Deneguem la resta d'ofertes
			$pending_offers = $this->Offer->find('all', array(
				'conditions' => array(
					'active' => true,
					'basket_id' => $offer['Offer']['basket_id'],
					'status' => Constants::STATE_ON_SALE,
				)
			));

			foreach($pending_offers as $key => $pending_offer) {
//				$pending_offers[$key]['Offer']['active'] = false;
				$pending_offers[$key]['Offer']['status'] = Constants::STATE_REFUSED;
			}

			$this->Offer->saveAll($pending_offers);
		}

		$durations = array('+1 week', '+2 week', '+1 month', '+2 month', '+3 month', '+6 month', '+1 year', '+2 year');

		$duration = $durations[$offer['Basket']['duration']];

		$win_amount = $offer['Basket']['amount']*($offer['Offer']['percentage']/100);

		$contract = $this->Contract->create();
		$contract['Contract']['basket_id'] = $offer['Basket']['id'];
		$contract['Contract']['shop_id'] = $offer['Shop']['id'];
		$contract['Contract']['begin_date'] = date('Y-m-d H:i:s');
		$contract['Contract']['end_date'] = date('Y-m-d H:i:s', strtotime($duration, strtotime($contract['Contract']['begin_date'])));
		$contract['Contract']['remaining_amount'] = $offer['Basket']['amount'] + $win_amount;
		$contract['Contract']['percentage'] = $offer['Offer']['percentage'];
		$contract['Contract']['active'] = true;

		if($this->Contract->save($contract)) {

			$client_account = $offer['Basket']['Client']['Account'];
			$client_account['locked_balance'] += $win_amount;

			if($this->Account->save($client_account)) {

				$shop_account = $offer['Shop']['Account'];
				$shop_account['locked_balance'] += $offer['Basket']['amount'];

				if($this->Account->save($shop_account)) {
					// Enviem la notificacio a la botiga
					$shop_user = $this->User->findById($offer['Shop']['user_id']);

					if($shop_user['User']['gcmRegId'] != null) {
						$this->sendPush($shop_user['User']['gcmRegId'],
							__("Felicitats!"),
							__('El client ').$user['User']['username'].__(' ha acceptat la teva oferta.'),
							Constants::NOTIFICATION_ACCEPTED_OFFER
						);
					}

					// Anticip directe
					$data = array();
					$data['user_id'] = $shop_user['User']['id'];
					$data['token'] = $shop_user['User']['token'];
					$data['contract_id'] = $this->Contract->getLastInsertID();
					$data['amount'] = $offer['Basket']['amount'];

					$Extracts = new ExtractsController;
					$Extracts->add($data);

					$this->set(array(
						'result' => Constants::RESULT_OK,
						'_serialize' => array('result')
					));
				} else {
					$this->set(array(
						'result' => Constants::ERROR_GENERIC,
						'message' => __('Ho sentim, no s\'ha pogut fer la transacció.'),
						'_serialize' => array('result', 'message')
					));
				}
			} else {
				$this->set(array(
					'result' => Constants::ERROR_GENERIC,
					'message' => __('Ho sentim, no s\'ha pogut fer la transacció.'),
					'_serialize' => array('result', 'message')
				));
			}
		} else {

			$this->Offer->id = $offer['Offer']['id'];
			$this->Offer->saveField('status', Constants::STATE_ON_SALE);

			$this->Basket->id = $offer['Basket']['id'];
			$this->Basket->saveField('state', Constants::STATE_SOLD);

			$this->set(array(
				'result' => Constants::ERROR_GENERIC,
				'message' => __('Ho sentim, no s\'ha pogut crear el contracte, torna-ho a provar siusplau.'),
				'_serialize' => array('result', 'message')
			));
		}

		$this->set(array(
			'result' => Constants::RESULT_OK,
			'contract' => $contract,
			'_serialize' => array('result', 'contract')
		));
	}

	public function deny($id) {

		$user_id = $this->request->data['user_id'];
		$token = $this->request->data['token'];

		$user = $this->User->find('first', array('conditions' => array('id' => $user_id, 'token' => $token)));

		if (empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		$offer = $this->Offer->findById($id);

		if(!empty($offer)) {
			$this->Offer->id = $offer['Offer']['id'];
			$this->Offer->saveField('status', Constants::STATE_REFUSED);

			$this->set(array(
				'result' => Constants::RESULT_OK,
				'_serialize' => array('result')
			));
		}
	}

	public function cancel($id) {

		$user_id = $this->request->data['user_id'];
		$token = $this->request->data['token'];

		$user = $this->User->find('first', array('conditions' => array('id' => $user_id, 'token' => $token)));

		if (empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		$offer = $this->Offer->findById($id);

		if(!empty($offer)) {
			$this->Offer->id = $offer['Offer']['id'];
			$this->Offer->saveField('status', Constants::STATE_CANCELED);

			$this->set(array(
				'result' => Constants::RESULT_OK,
				'_serialize' => array('result')
			));
		}
	}
}
