<?php
App::uses('AppController', 'Controller');
/**
 * Sales Controller
 *
 */
class SalesController extends AppController {

	/**
	 * Scaffold
	 *
	 * @var mixed
	 */
	public $scaffold;

	public $components = array('RequestHandler');
	public $uses = array('Sale', 'User', 'Contract', 'Shop');

	public function index()
	{
		$user_id = $this->request->query['user_id'];
		$token = $this->request->query['token'];

		$user = $this->User->find('first', array('conditions' => array('id' => $user_id, 'token' => $token)));

		if (empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		$this->Sale->recursive = -1;
		$sales = $this->Sale->find('all', array(
			'conditions' => array(
				'contract_id' => $this->request->query['contract_id'],
				'active' => true,
				'state' => 1
			),
			'order' => array('Sale.created desc')
		));

		$this->set(array(
			'result' => Constants::RESULT_OK,
			'sales' => $sales,
			'_serialize' => array('result', 'sales')
		));
	}

	public function add() {
		$user_id = $this->request->data['user_id'];
		$token = $this->request->data['token'];

		$user = $this->User->find('first', array('conditions' => array('id' => $user_id, 'token' => $token)));

		if (empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		$contract_id = $this->request->data['contract_id'];
		$amount = $this->request->data['amount'];
		$concept = $this->request->data['concept'];

		if(isset($amount) && isset($concept) && $amount != 0 && $concept != "") {
			$contract = $this->Contract->findById($contract_id);
			if ($amount > $contract['Contract']['remaining_amount']) {
				$this->set(array(
					'result' => Constants::ERROR_AMOUNT,
					'message' => 'L\'import introduit no pot superar la quantitat restant a la cistella (' . $contract['Contract']['remaining_amount'] . ') eurakos.',
					'_serialize' => array('result', 'message')
				));
				return;
			}

			$sale = $this->Sale->create();
			$sale['Sale']['contract_id'] = $contract_id;
			$sale['Sale']['amount'] = $amount;
			$sale['Sale']['concept'] = $concept;
			$sale['Sale']['state'] = Constants::SALE_STATE_WAITING;
			$sale['Sale']['active'] = true;

			if ($this->Sale->save($sale)) {

				// Enviem la notificacio a l'usuari de la cistella
				$this->Contract->recursive = -1;
				$this->Contract->Behaviors->load('Containable');
				$contract = $this->Contract->find('first', array(
						'conditions' => array('Contract.id' => $contract_id),
						'contain' => array('Basket.Client.User')
					)
				);

				if ($contract['Basket']['Client']['User']['gcmRegId'] != null) {
					$this->sendPush($contract['Basket']['Client']['User']['gcmRegId'],
						__("S'ha iniciat una compra"),
						__('Una botiga acaba d\'iniciar un procés de compra amb tu, entra per continuar.'),
						Constants::NOTIFICATION_SALE_CONFIRMATION,
						$this->Sale->getLastInsertID()
					);
				}


				$this->set(array(
					'result' => Constants::RESULT_OK,
					'_serialize' => array('result')
				));
			} else {
				$this->set(array(
					'result' => Constants::ERROR_GENERIC,
					'message' => __('Ho sentim, no s\'ha pogut crear la venda, torna-ho a provar siusplau.'),
					'_serialize' => array('result', 'message')
				));
			}
		} else {
			$this->set(array(
				'result' => Constants::ERROR_GENERIC,
				'message' => __('Has d\'introduir el concepte i l\'import de la compra.'),
				'_serialize' => array('result', 'message')
			));
		}
	}

	public function view($id)
	{
		$user_id = $this->request->query['user_id'];
		$token = $this->request->query['token'];

		$user = $this->User->find('first', array('conditions' => array('id' => $user_id, 'token' => $token)));

		if (empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		$this->Sale->recursive = -1;
		$this->Sale->Behaviors->load('Containable');
		$sale = $this->Sale->find('first', array(
			'conditions' => array(
				'Sale.id' => $id,
				'Sale.active' => true,
			),
			'contain' => array('Contract.Shop.name', 'Contract.percentage')
		));

		$this->set(array(
			'result' => Constants::RESULT_OK,
			'sale' => $sale,
			'_serialize' => array('result', 'sale')
		));
	}

	public function accept($id) {
		$user_id = $this->request->data['user_id'];
		$token = $this->request->data['token'];

		$this->User->recursive = -1;
		$this->User->Behaviors->load('Containable');
		$user = $this->User->find('first',
			array(
				'conditions' => array('id' => $user_id, 'token' => $token),
			)
		);

		if (empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		$result = $this->Sale->accept($id);

		if($result == Constants::RESULT_OK) {
			$this->Sale->id = $id;
			$this->Sale->saveField('state', Constants::SALE_STATE_ACCEPTED);

			$this->set(array(
				'result' => Constants::RESULT_OK,
				'_serialize' => array('result')
			));

			// Enviem la notificacio a la botiga
			$this->Shop->recursive = -1;
			$this->Shop->Behaviors->load('Containable');
			$shop = $this->Shop->find('first', array(
				'joins' => array(
					array(
						'table' => 'contract',
						'alias' => 'Contract',
						'conditions' => array(
							'Contract.shop_id = Shop.id'
						)
					),
					array(
						'table' => 'sale',
						'alias' => 'Sale',
						'conditions' => array(
							'Sale.contract_id = Contract.id',
							'Sale.id' => $id
						)
					)
				),
				'conditions' => array(),
				'contain' => array('User')
			));

			if($shop['User']['gcmRegId'] != null) {
				$this->sendPush($shop['User']['gcmRegId'],
					__("Compra realitzada!"),
					__('El client ').$user['User']['username'].__(' ha acceptat la transacció.'),
					Constants::NOTIFICATION_ACCEPTED_SALE
				);
			}

		} else if($result == Constants::ERROR_AMOUNT) {
			$this->set(array(
				'result' => Constants::ERROR_AMOUNT,
				'message' => __('La quantitat que has entrat es superior a la permesa.'),
				'_serialize' => array('result', 'message')
			));
		} else {
			$this->set(array(
				'result' => Constants::ERROR_GENERIC,
				'message' => __('Ho sentim, no s\'ha pogut fer la transacció.'),
				'_serialize' => array('result', 'message')
			));
		}
	}

	public function deny($id) {
		$user_id = $this->request->data['user_id'];
		$token = $this->request->data['token'];

		$this->User->recursive = -1;
		$this->User->Behaviors->load('Containable');
		$user = $this->User->find('first',
			array(
				'conditions' => array('id' => $user_id, 'token' => $token),
			)
		);

		if (empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		$this->Sale->id = $id;
		$this->Sale->saveField('state', Constants::SALE_STATE_DENIED);

		$this->set(array(
			'result' => Constants::RESULT_OK,
			'_serialize' => array('result')
		));

		// Enviem la notificacio a la botiga
		$this->Shop->recursive = -1;
		$this->Shop->Behaviors->load('Containable');
		$shop = $this->Shop->find('first', array(
			'joins' => array(
				array(
					'table' => 'contract',
					'alias' => 'Contract',
					'conditions' => array(
						'Contract.shop_id = Shop.id'
					)
				),
				array(
					'table' => 'sale',
					'alias' => 'Sale',
					'conditions' => array(
						'Sale.contract_id = Contract.id',
						'Sale.id' => $id
					)
				)
			),
			'conditions' => array(),
			'contain' => array('User')
		));

		if($shop['User']['gcmRegId'] != null) {
			$this->sendPush($shop['User']['gcmRegId'],
				__("Ho sentim"),
				__('El client ').$user['User']['username'].__(' no ha acceptat la transacció, posa\'t en contacte amb ell per saber que ha passat.'),
				Constants::NOTIFICATION_DENIED_SALE
			);
		}
	}
}
