<?php
App::uses('AppController', 'Controller');
/**
 * Shops Controller
 *
 */
class ShopsController extends AppController {

/**
 * Scaffold
 *
 * @var mixed
 */
	public $components = array('RequestHandler');

	public $scaffold;

	public  $uses = array('Shop', 'User', 'Account', 'Client', 'ActivityShop');

	public function beforeFilter() {
		parent::beforeFilter();
		if($this->request->action != 'index') {
			$user = $this->Auth->User();
			if (!isset($user['role']) || $user['role'] != Constants::ADMIN) {
				$this->redirect(array(
					'controller' => 'users',
					'action' => 'login'
				));
			}
		}
	}

	public function index() {
		if($this->request->is('get')) {
			$user_id = $this->request->query['user_id'];
			$token = $this->request->query['token'];

			$user = $this->User->find('all', array('conditions' => array('id' => $user_id, 'token' => $token)));

			if(empty($user)) {
				$this->set(array(
					'result' => Constants::ERROR_GENERIC,
					'message' => 'Error authenticating user.',
					'_serialize' => array('result', 'message')
				));
				return;
			}

			$this->Shop->recursive = -1;
			$this->Shop->Behaviors->load('Containable');
			$shops = $this->Shop->find('all', array(
				'conditions' => array('Shop.active' => true),
				'contain' => array('Activity', 'User.email')
			));

			// TODO: calcular distancia
			foreach($shops as $key => $shop) {
				$shops[$key]['Shop']['distance'] = 0;
			}

			$this->set(array(
				'result' => Constants::RESULT_OK,
				'shops' => $shops,
				'_serialize' => array('result', 'shops')
			));
		}
	}

	public function view_page($id) {
		$this->set('shop', $this->Shop->findById($id));
	}

	public function view($id) {
		if($this->request->is('get')) {
			$user_id = $this->request->query['user_id'];
			$token = $this->request->query['token'];

			$user = $this->User->find('all', array('conditions' => array('id' => $user_id, 'token' => $token)));

			if(empty($user)) {
				$this->set(array(
					'result' => Constants::ERROR_GENERIC,
					'message' => 'Error authenticating user.',
					'_serialize' => array('result', 'message')
				));
				return;
			}

			$this->Shop->recursive = -1;
			$this->Shop->Behaviors->load('Containable');
			$shop = $this->Shop->find('first', array(
				'conditions' => array('Shop.active' => true),
				'contain' => array('Activity', 'User.email')
			));

			$this->set(array(
				'result' => Constants::RESULT_OK,
				'shop' => $shop,
				'_serialize' => array('result', 'shops')
			));
		}
	}

	public function index_page() {
		$this->Shop->recursive = 0;
		$this->set('shops', $this->paginate());
	}

	public function add() {

		$this->set('users', $this->Shop->User->find('list'));
		$this->set('activities', $this->Shop->Activity->find('list'));

		if ($this->request->is('post')) {

			$error = false;

			// Creem l'usuari i la seva compta
			$this->User->create();
			if ($this->User->save($this->request->data['User'])) {
				$account = $this->Account->create();
				$account['balance'] = 0;
				$account['locked_balance'] = 0;
				$account['active'] = true;

				if ($this->Account->save($account)) {
					$client = $this->Client->create();
					$client['Client']['user_id'] = $this->User->getLastInsertId();
					$client['Client']['account_id'] = $this->Account->getLastInsertId();
					$client['Client']['active'] = true;

					if (!$this->Client->save($client)) {
						$this->Account->delete($client['Client']['account_id']);
						$this->User->delete($client['Client']['user_id']);
						$error = true;
					}
				} else {
					$this->User->delete($this->User->getLastInsertId());
					$error = true;
				}
			} else {
				$error = true;
			}

			$this->request->data['Shop']['active'] = true;

			// Creem la botiga i la seva compta
			if(!$error) {
				$account = $this->Account->create();
				$account['Account']['balance'] = 0;
				$account['Account']['locked_balance'] = 0;
				$account['active'] = true;
				if ($this->Shop->Account->saveAll($account)) {
					$account_id = $this->Shop->Account->getLastInsertID();
					$this->request->data['Shop']['account_id'] = $account_id;
					$this->request->data['Shop']['user_id'] = $this->User->getLastInsertId();

					$this->Shop->create();
					if ($this->Shop->save($this->request->data['Shop'])) {
						$shop_id = $this->Shop->getLastInsertID();

						$index = 0;
						$activity_shop = array();
						foreach($this->request->data['Activity']['Activity'] as $activity) {
							$activity_shop[$index]['ActivityShop']['shop_id'] = $shop_id;
							$activity_shop[$index]['ActivityShop']['activity_id'] = $activity;
							$index++;
						}

						CakeLog::debug("ActivityShop: ".print_r($activity_shop,true));

						if($this->ActivityShop->saveAll($activity_shop)) {
							$this->Session->setFlash(__('The shop has been saved.'));
							return $this->redirect(array('action' => 'index_page'));
						} else {
							$this->Session->setFlash(__('The shop could not be saved. Please, try again.'));
						}
					} else {

						$this->Session->setFlash(__('The shop could not be saved. Please, try again.'));
					}
				} else {
					$this->Session->setFlash(__('The shop could not be saved. Please, try again.'));
				}
			} else {
				$this->Session->setFlash(__('The shop could not be saved. Please, try again.'));
			}
		}
	}
}
