<?php
App::uses('AppController', 'Controller');
App::uses('SimplePasswordHasher', 'Controller/Component/Auth');
/**
 * Users Controller
 *
 */
class UsersController extends AppController {

	public $components = array('RequestHandler', 'Paginator');

	public $uses = array('User', 'Account', 'Client', 'Shop');

	public $paginate = array(
		'limit' => 25,
		'fields' => array('User.id', 'User.username', 'User.email', 'User.role', 'User.created', 'User.modified')
	);

	public function beforeFilter() {
		parent::beforeFilter();
		$this->Auth->allow('login', 'logout');

		if(in_array($this->request->action, array('index_page', 'view'))) {
			$user = $this->Auth->User();
			if (!isset($user['role']) || $user['role'] != Constants::ADMIN) {
				$this->redirect(array(
					'controller' => 'users',
					'action' => 'login'
				));
			}
		}
	}

	public function add()
	{
		// Check if username and email alreaady exists in DB
//		$username_bd = $this->User->findByUsername($this->request->data['username']);
		$email_bd = $this->User->findByEmail($this->request->data['email']);

		 if(!empty($email_bd)){
			$result = Constants::ERROR_REPEAT_EMAIL;
			$message = 'Email already exsists';
		} else {
			$this->User->create();
			if ($this->User->save($this->request->data)) {

				$account = $this->Account->create();
				$account['balance'] = 0;
				$account['locked_balance'] = 0;
				$account['active'] = true;


				if ($this->Account->save($account)) {

					$client = $this->Client->create();
					$client['Client']['user_id'] = $this->User->getLastInsertId();
					$client['Client']['account_id'] = $this->Account->getLastInsertId();
					$client['Client']['active'] = true;

					if ($this->Client->save($client)) {
						$result = Constants::RESULT_OK;
						$message = 'Saved';
					} else {

						$this->Account->delete($client['Client']['account_id']);
						$this->User->delete($client['Client']['user_id']);

						$result = Constants::ERROR_GENERIC;
						$message = 'Error saving the client';
					}
				} else {
					$this->User->delete($this->User->getLastInsertId());

					$result = Constants::ERROR_GENERIC;
					$message = 'Error saving the account';
				}
			} else {
				$result = Constants::ERROR_GENERIC;
				$message = 'Error saving the user';
			}
		}
		$this->set(array(
			'result' => $result,
			'message' => $message,
			'_serialize' => array('result','message')
		));
	}

	public function index_page() {
		$this->Paginator->settings = $this->paginate;
		$this->set('users', $this->Paginator->paginate('User'));

	}

	public function user_login() {

		$email = $this->data['email'];

		// Hash password to check if it's the same as DB
		$passwordHasher = new SimplePasswordHasher(array('hashType' => 'sha256'));
		$password = $passwordHasher->hash($this->data['password']);

		$user = $this->User->findByEmail($email);

		if(!empty($user)) {

			$shop = $this->Shop->findByUserId($user['User']['id']);
			if(!empty($shop)) {
				$shop = $shop['Shop']['id'];
			} else {
				$shop = -1;
			}

			if($user['User']['password'] == $password) {

				$token = $passwordHasher->hash($email.date('dmY'));

				$this->User->id = $user['User']['id'];
				$this->User->saveField('token', $token);

				if(isset($this->data['regid'])) {
					$this->User->id = $user['User']['id'];
					$this->User->saveField('gcmRegId', $this->data['regid']);
				}

				$this->set(array(
					'result' => Constants::RESULT_OK,
					'id' => $user['User']['id'],
					'email' => $user['User']['email'],
					'shop' => $shop,
					'token' => $token,
					'username' => $user['User']['username'],
					'_serialize' => array('result', 'id', 'token','email', 'shop', 'username')
				));
			} else {
				$this->set(array(
					'result' => Constants::ERROR_INCORRECT_PASSWORD,
					'message' => 'Incorrect password.',
					'_serialize' => array('result', 'message')
				));
			}
		} else {
			$this->set(array(
				'result' => Constants::ERROR_UNEXISTING_USERNAME,
				'message' => 'Incorrect username.',
				'_serialize' => array('result', 'message')
			));
		}
	}

	public function login() {
		if ($this->request->is('post')) {
			if ($this->Auth->login()) {
				return $this->redirect($this->Auth->redirectUrl());
			}
			$this->Session->setFlash(__('Invalid email or password, try again'));
		}
	}

	public function logout() {
		return $this->redirect($this->Auth->logout());
	}

	public function error_version() {
		$this->set(array(
			'result' => Constants::ERROR_VERSION,
			'_serialize' => array('result')
		));
	}

}
