<?php
App::uses('AppController', 'Controller');
App::uses('CakeEmail', 'Network/Email');
/**
 * Withdraws Controller
 *
 */
class WithdrawsController extends AppController {

	public $components = array('RequestHandler', 'Paginator');
	public $uses = array('Withdraw', 'User', 'Account');

	public $paginate = array(
		'limit' => 20,
		'order' => array(
			'Withdraw.created' => 'desc'
		),
		'contain' => array(
			'Account.Shop.User', 'Account.Client.User'
		)
	);

	public function beforeFilter() {
		parent::beforeFilter();
		if(in_array($this->request->action, array('index','markAsDone','cancel'))) {
			$user = $this->Auth->User();
			if (!isset($user['role']) || $user['role'] != Constants::ADMIN) {
				$this->redirect(array(
					'controller' => 'users',
					'action' => 'login'
				));
			}
		}
	}

	public function index() {
		$this->Paginator->settings = $this->paginate;
		$this->Withdraw->Behaviors->load('Containable');
		$this->set('withdraws', $this->paginate());
	}

	public function add($data) {

		if(!empty($this->request->data['user_id'])) {
			$data = $this->request->data;
		}

		$user_id = $data['user_id'];
		$token = $data['token'];

		$this->User->recursive = -1;
		$this->User->Behaviors->load('Containable');

		if(isset($data['is_shop']) && $data['is_shop']) {
			$user = $this->User->find('first',
				array(
					'conditions' => array('id' => $user_id, 'token' => $token),
					'contain' => array('Shop', 'Shop.Account')
				)
			);
		} else {
			$user = $this->User->find('first',
				array(
					'conditions' => array('id' => $user_id, 'token' => $token),
					'contain' => array('Client', 'Client.Account')
				)
			);
		}

		if(empty($user)) {
			$this->set(array(
				'result' => Constants::ERROR_LOGIN,
				'message' => 'Error authenticating user.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		if(!isset($data['amount']) || $data['amount'] <= 0) {
			$this->set(array(
				'result' => Constants::ERROR_AMOUNT,
				'message' => 'Has d\'entrar una quantitat a retirar.',
				'_serialize' => array('result', 'message')
			));
			return;
		}

		if(isset($data['is_shop']) && $data['is_shop']) {
			$account = $user['Shop'][0]['Account'];
		} else {
			$account = $user['Client'][0]['Account'];
		}

		if($account['balance'] < $data['amount']) {
			$this->set(array(
				'result' => Constants::ERROR_AMOUNT,
				'message' => __('No pots retirar una quantitat superior al teu saldo (%s eurakos).', $account['balance']),
				'_serialize' => array('result', 'message')

			));
			return;
		}

//		$account['balance'] -= $data['amount'];

		if($this->Account->save($account)) {

			$withdraw = $this->Withdraw->create();

			$withdraw['Withdraw']['account_id'] = $account['id'];
			$withdraw['Withdraw']['amount'] = $data['amount'];
			$withdraw['Withdraw']['bank_account'] = $data['bank_account'];
			$withdraw['Withdraw']['status'] = Constants::WITHDRAW_STATUS_PENDING;
			$withdraw['Withdraw']['active'] = 1;

			if($this->Withdraw->save($withdraw)) {

				// Enviem el correu de notifiació
//				$Email = new CakeEmail('default');
//				$Email->from(array('kevingarcia@easyinnova.com' => 'Kevin'));
//				$Email->to('kevingarcia@easyinnova.com');
//				$Email->subject('Nou anticip pendent');
//				$Email->send('Hi ha un nou anticip pendent de fer.');

				$this->set(array(
					'result' => Constants::RESULT_OK,
					'_serialize' => array('result')
				));
			} else {

				$account['balance'] += $data['amount'];

				$this->Account->save($account);
				$this->set(array(
					'result' => Constants::ERROR_GENERIC,
					'message' => 'Hi ha hagut un error, torna a provar si us plau.',
					'_serialize' => array('result', 'message')
				));
			}
		}
	}

	public function markAsDone($id) {
		$this->Withdraw->id = $id;
		$this->Withdraw->saveField('status', Constants::WITHDRAW_STATUS_DONE);

		$this->redirect(array('action' => 'index'));
	}

	public function cancel($id) {
		$this->Withdraw->id = $id;
		$this->Withdraw->saveField('status', Constants::WITHDRAW_STATUS_CANCELED);

		$this->redirect(array('action' => 'index'));
	}

}
