<?php
App::uses('AppModel', 'Model');
/**
 * Sale Model
 *
 * @property Contract $Contract
 */
class Sale extends AppModel {

/**
 * Use table
 *
 * @var mixed False or table name
 */
	public $useTable = 'sale';

/**
 * Validation rules
 *
 * @var array
 */
	public $validate = array(
		'contract_id' => array(
			'numeric' => array(
				'rule' => array('numeric'),
				//'message' => 'Your custom message here',
				//'allowEmpty' => false,
				//'required' => false,
				//'last' => false, // Stop validation after this rule
				//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
	);

	//The Associations below have been created with all possible keys, those that are not needed can be removed

/**
 * belongsTo associations
 *
 * @var array
 */
	public $belongsTo = array(
		'Contract' => array(
			'className' => 'Contract',
			'foreignKey' => 'contract_id',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		)
	);

	public function accept($sale_id) {

		$this->recursive = -1;
		$this->Behaviors->load('Containable');
		$sale = $this->find('first', array(
			'conditions' => array(
				'Sale.id' => $sale_id,
				'Sale.active' => true,
				'Sale.state' => Constants::STATE_ON_SALE
			),
			'contain' => array('Contract.Basket.Client.Account', 'Contract.Shop.Account')
		));

		if(!empty($sale)) {
			$amount = $sale['Sale']['amount'];
			$percentage = $sale['Contract']['percentage'];
			$rate = 100 / (100 + $percentage);

			if ($sale['Contract']['remaining_amount'] < $amount) {
				return Constants::ERROR_AMOUNT;
			}

			// Comprovem si hi ha algun anticip pendent
			$extract = $this->Contract->Extract->find('first', array(
				'conditions' => array(
					'contract_id' => $sale['Contract']['id'],
					'active' => true,
				)
			));

			$contract = $sale['Contract'];
			$contract['remaining_amount'] -= $amount;

			$from_account = $sale['Contract']['Basket']['Client']['Account'];
			$to_account = $sale['Contract']['Shop']['Account'];

			// Descomptem del compte del client
			$from_account['locked_balance'] -= $amount;

			// Afegim al compte del botiguer o ho restem de l'anticip
			if (!empty($extract)) {
				if ($extract['Extract']['remaining_to_pay'] >= $amount * $rate) {
					$extract['Extract']['remaining_to_pay'] -= $amount * $rate;
				} else {
					$to_account['locked_balance'] -= ($amount * $rate - $extract['Extract']['remaining_to_pay'] * $rate);
					$to_account['balance'] += ($amount * $rate - $extract['Extract']['remaining_to_pay'] * $rate);
					$extract['Extract']['remaining_to_pay'] = 0;
				}
			} else {
				$to_account['locked_balance'] -= $amount * $rate;
				$to_account['balance'] += $amount * $rate;
			}

			$this->begin();

			$result = Constants::RESULT_OK;
			if ($this->Contract->Basket->Client->Account->save($from_account)) {
				if ($this->Contract->Shop->Account->save($to_account)) {
					if ($this->Contract->save($contract)) {
						if (!empty($extract)) {
							if ($this->Contract->Extract->save($extract)) {
								$this->commit();
							} else {
								$result = Constants::ERROR_GENERIC;
								$this->rollback();
							}
						} else {
							$this->commit();
						}
					} else {
						$result = Constants::ERROR_GENERIC;
						$this->rollback();
					}
				} else {
					$result = Constants::ERROR_GENERIC;
					$this->rollback();
				}
			} else {
				$result = Constants::ERROR_GENERIC;
				$this->rollback();
			}
		} else {
			$result = Constants::ERROR_GENERIC;
		}

		return $result;
	}
}
